#![cfg(test)]

use std::collections::HashMap;

use super::*;
use list::*;

#[test]
fn add_action() {
    let mut at = ActionTrust::default();

    at.add_action(Action::Block, 123)
        .add_action(Action::Silence, 456);

    let test_at = ActionTrust {
        block: 123,
        silence: 456,
    };

    assert_eq!(at, test_at);
}

#[test]
fn add_action_overlap() {
    let mut at = ActionTrust::default();

    at.add_action(Action::Block, 123)
        .add_action(Action::Block, 333)
        .add_action(Action::Silence, 123);

    let test_at = ActionTrust {
        block: 456,
        silence: 123,
    };

    assert_eq!(at, test_at);
}

#[test]
fn source_new_from_hashmap() {
    let src1 = Source::from(HashMap::from([
        (String::from("example.com"), Action::Block),
        (String::from("example.org"), Action::Silence),
        (String::from("example.net"), Action::Block),
    ]));

    assert_eq!(
        src1.actions,
        HashMap::from([
            (String::from("example.com"), Action::Block),
            (String::from("example.org"), Action::Silence),
            (String::from("example.net"), Action::Block),
        ])
    );

    assert_eq!(src1.trust, 100);
}

#[test]
fn source_build_from_hashmap() {
    let src2 = Source::build(
        HashMap::from([
            (String::from("example.com"), Action::Block),
            (String::from("example.org"), Action::Silence),
            (String::from("example.net"), Action::Block),
        ]),
        123,
    );

    assert_eq!(
        src2.actions,
        HashMap::from([
            (String::from("example.com"), Action::Block),
            (String::from("example.org"), Action::Silence),
            (String::from("example.net"), Action::Block),
        ])
    );
    assert_eq!(src2.trust, 123);
}

#[test]
fn build_moderation_list() {
    let mut ml = ModerationList::default();

    let src1 = Source::from(HashMap::from([
        (String::from("example.com"), Action::Block),
        (String::from("example.org"), Action::Silence),
        (String::from("example.net"), Action::Block),
    ]));

    let src2 = Source::from(HashMap::from([
        (String::from("example.com"), Action::Block),
        (String::from("example.org"), Action::Block),
        (String::from("example.net"), Action::Silence),
    ]));

    ml.add_source(src1).add_source(src2);

    let test_ml = ModerationList(
        HashMap::from([
            (String::from("example.com"), ActionTrust::from((200, 0))),
            (String::from("example.org"), ActionTrust::from((100, 100))),
            (String::from("example.net"), ActionTrust::from((100, 100))),
        ])
    );

    assert_eq!(ml, test_ml);

    let src3 = Source::build(HashMap::from([
        (String::from("example.com"), Action::Block),
        (String::from("example.org"), Action::Silence),
    ]), 200);

    let src4 = Source::build(HashMap::from([
        (String::from("example.com"), Action::Block),
        (String::from("example.net"), Action::Silence),
    ]), 50);

    ml.add_source(src3).add_source(src4);

    let test_ml = ModerationList(
        HashMap::from([
            (String::from("example.com"), ActionTrust::from((450, 0))),
            (String::from("example.org"), ActionTrust::from((100, 300))),
            (String::from("example.net"), ActionTrust::from((100, 150))),
        ])
    );

    assert_eq!(ml, test_ml);

}
